@@ -454,14 +454,48 @@ static void setup_log_ctx(gnb_conf_t *conf, gnb_log_ctx_t *log){
 
 }
 
+void update_node_crypto_key(gnb_core_t *gnb_core, uint64_t now_sec){
+
+    int need_update_time_seed;
+
+    size_t num = gnb_core->ctl_block->node_zone->node_num;
+
+    if ( 0 == num ) {
+        return;
+    }
+
+    need_update_time_seed = gnb_verify_seed_time(gnb_core, now_sec);
+
+    if ( 0 == need_update_time_seed ) {
+        return;
+    }
+
+    gnb_update_time_seed(gnb_core, now_sec);
+
+    int i;
+
+    gnb_node_t *node;
+
+    for ( i=0; i<num; i++ ) {
+
+        node = &gnb_core->ctl_block->node_zone->node[i];
+
+        if ( gnb_core->local_node->uuid64 == node->uuid64 ) {
+            continue;
+        }
+
+        gnb_build_crypto_key(gnb_core, node);
+
+    }
+
+}
+
 
 gnb_core_t* gnb_core_create(gnb_conf_t *conf){
 
     gnb_core_t *gnb_core;
 
-    gnb_heap_t *heap = gnb_heap_create(8192);
-
-    int i;
+    gnb_heap_t *heap = gnb_heap_create(conf->max_heap_fragment);
 
     gnb_core = gnb_heap_alloc(heap, sizeof(gnb_core_t));
     memset(gnb_core, 0, sizeof(gnb_core_t));
@@ -493,7 +527,7 @@ gnb_core_t* gnb_core_create(gnb_conf_t *conf){
     gnb_core->ifname = (char *)gnb_core->ctl_block->core_zone->ifname;
     gnb_core->if_device_string = (char *)gnb_core->ctl_block->core_zone->if_device_string;
 
-    gnb_core->uuid_node_map   = gnb_hash32_create(gnb_core->heap, 1024, 1024); //以节点的uuid32作为key的 node 表
+    gnb_core->uuid_node_map   = gnb_hash32_create(gnb_core->heap, 1024, 1024); //以节点的uuid64作为key的 node 表
     gnb_core->ipv4_node_map   = gnb_hash32_create(gnb_core->heap, 1024, 1024);
 
     //以节点的subnet(uint32)作为key的 node 表
@@ -630,7 +664,13 @@ gnb_core_t* gnb_core_create(gnb_conf_t *conf){
     }
 
     if ( gnb_core->conf->activate_index_worker ) {
-        gnb_core->index_worker  = gnb_worker_init("gnb_index_worker", gnb_core);
+
+        if ( 1 == gnb_core->conf->safe_index && 0 == gnb_core->conf->lite_mode ) {
+            gnb_core->index_worker  = gnb_worker_init("gnb_secure_index_worker", gnb_core);
+        } else {
+            gnb_core->index_worker  = gnb_worker_init("gnb_index_worker", gnb_core);
+        }
+
     }
 
     if ( gnb_core->conf->activate_detect_worker ) {
@@ -638,9 +678,14 @@ gnb_core_t* gnb_core_create(gnb_conf_t *conf){
     }
 
     if ( gnb_core->conf->activate_index_service_worker ) {
-        gnb_core->index_service_worker  = gnb_worker_init("gnb_index_service_worker", gnb_core);
-    }
 
+        if ( 1 == gnb_core->conf->safe_index ) {
+            gnb_core->index_service_worker  = gnb_worker_init("gnb_secure_index_service_worker", gnb_core);
+        } else {
+            gnb_core->index_service_worker  = gnb_worker_init("gnb_index_service_worker", gnb_core);            
+        }
+
+    }
 
     if ( gnb_core->conf->pf_worker_num > 0 ) {
 
@@ -790,6 +835,8 @@ void gnb_core_start(gnb_core_t *gnb_core){
 
     gnb_setup_env(gnb_core);
 
+    update_node_crypto_key(gnb_core, 1);
+
     if ( gnb_core->conf->activate_tun ) {
 
         ret = gnb_core->drv->open_tun(gnb_core);
@@ -806,7 +853,7 @@ void gnb_core_start(gnb_core_t *gnb_core){
         }
 
         GNB_LOG1(gnb_core->log, GNB_LOG_ID_CORE,"if[%s] opened\n", gnb_core->ifname);
-        GNB_LOG1(gnb_core->log, GNB_LOG_ID_CORE,"node[%d] ipv4[%s]\n", gnb_core->local_node->uuid32, GNB_ADDR4STR_PLAINTEXT1(&gnb_core->local_node->tun_addr4));
+        GNB_LOG1(gnb_core->log, GNB_LOG_ID_CORE,"node[%llu] ipv4[%s]\n", gnb_core->local_node->uuid64, GNB_ADDR4STR_PLAINTEXT1(&gnb_core->local_node->tun_addr4));
 
         for ( i=0; i<gnb_core->pf_worker_ring->size; i++ ) {
             gnb_core->pf_worker_ring->worker[i]->start(gnb_core->pf_worker_ring->worker[i]);
