@@ -32,7 +32,6 @@
 #include <pthread.h>
 #include <unistd.h>
 
-#include "ed25519/ed25519.h"
 
 #include "gnb.h"
 #include "gnb_node.h"
@@ -42,7 +41,7 @@
 #include "gnb_binary.h"
 #include "gnb_worker_queue_data.h"
 #include "gnb_index_frame_type.h"
-
+#include "ed25519/ed25519.h"
 
 typedef struct _index_worker_ctx_t {
 
@@ -51,10 +50,8 @@ typedef struct _index_worker_ctx_t {
     gnb_payload16_t *index_frame_payload;
 
     struct timeval now_timeval;
-
     uint64_t now_time_sec;
     uint64_t now_time_usec;
-
     uint64_t last_post_addr_ts_sec;
 
     pthread_t thread_worker;
@@ -80,8 +77,7 @@ static void send_post_addr_frame(gnb_worker_t *gnb_index_worker){
 
     memcpy(post_addr_frame->data.src_key512, gnb_core->local_node->key512, 64);
 
-    post_addr_frame->data.src_uuid32 = htonl(gnb_core->local_node->uuid32);
-
+    post_addr_frame->data.src_uuid64  = gnb_htonll(gnb_core->local_node->uuid64);
     post_addr_frame->data.src_ts_usec = gnb_htonll(index_worker_ctx->now_time_usec);
 
     if ( 0 != gnb_core->ctl_block->core_zone->wan6_port ) {
@@ -91,11 +87,7 @@ static void send_post_addr_frame(gnb_worker_t *gnb_index_worker){
     }
 
     //debug_text
-    snprintf(post_addr_frame->data.text,32,"[%u]POST ADDR",gnb_core->local_node->uuid32);
-
-    if ( 0 == gnb_core->conf->lite_mode ) {
-        ed25519_sign(post_addr_frame->src_sign, (const unsigned char *)&post_addr_frame->data, sizeof(struct post_addr_frame_data), gnb_core->ed25519_public_key, gnb_core->ed25519_private_key);
-    }
+    snprintf(post_addr_frame->data.text, 32, "[%llu]POST ADDR", gnb_core->local_node->uuid64);
 
     //向所有index节点post
     gnb_send_address_list(gnb_core, gnb_core->index_address_ring.address_list, index_worker_ctx->index_frame_payload);
@@ -123,7 +115,7 @@ static void send_request_addr_frame(gnb_worker_t *gnb_index_worker, gnb_node_t *
 
     index_worker_ctx->index_frame_payload->sub_type = PAYLOAD_SUB_TYPE_REQUEST_ADDR;
 
-    gnb_payload16_set_data_len( index_worker_ctx->index_frame_payload,  sizeof(request_addr_frame_t) );
+    gnb_payload16_set_data_len( index_worker_ctx->index_frame_payload, sizeof(request_addr_frame_t) );
 
     request_addr_frame_t *request_addr_frame = (request_addr_frame_t *)index_worker_ctx->index_frame_payload->data;
     memset(request_addr_frame, 0, sizeof(request_addr_frame_t));
@@ -135,17 +127,12 @@ static void send_request_addr_frame(gnb_worker_t *gnb_index_worker, gnb_node_t *
     memcpy(request_addr_frame->data.src_key512, gnb_core->local_node->key512, 64);
     memcpy(request_addr_frame->data.dst_key512, node->key512, 64);
 
-    request_addr_frame->data.src_uuid32 = htonl(gnb_core->local_node->uuid32);
-    request_addr_frame->data.dst_uuid32 = htonl(node->uuid32);
-
+    request_addr_frame->data.src_uuid64  = gnb_htonll(gnb_core->local_node->uuid64);
+    request_addr_frame->data.dst_uuid64  = gnb_htonll(node->uuid64);
     request_addr_frame->data.src_ts_usec = gnb_htonll(index_worker_ctx->now_time_usec);
 
     //debug_text
-    snprintf(request_addr_frame->data.text,32,"%s %u==>%u","REQUEST_ADDR", gnb_core->local_node->uuid32, node->uuid32);
-
-    if (0 == gnb_core->conf->lite_mode) {
-        ed25519_sign(request_addr_frame->src_sign, (const unsigned char *)&request_addr_frame->data, sizeof(struct request_addr_frame_data), gnb_core->ed25519_public_key, gnb_core->ed25519_private_key);
-    }
+    snprintf(request_addr_frame->data.text,32, "%s %llu==>%llu", "REQUEST_ADDR", gnb_core->local_node->uuid64, node->uuid64);
 
     gnb_address_t *address;
 
@@ -154,24 +141,23 @@ static void send_request_addr_frame(gnb_worker_t *gnb_index_worker, gnb_node_t *
         address = gnb_select_index_address(gnb_core, index_worker_ctx->now_time_sec);
 
         if (NULL!=address) {
-            //gnb_send_to_address(gnb_core, address, index_worker_ctx->index_frame_payload);
-            gnb_send_to_address_through_all_sockets(gnb_core, address, index_worker_ctx->index_frame_payload);
+            gnb_send_to_address_through_all_sockets(gnb_core, address, index_worker_ctx->index_frame_payload,1);
         }
 
     } else {
         //向所有index节点发请求
-        gnb_send_address_list_through_all_sockets(gnb_core, gnb_core->index_address_ring.address_list, index_worker_ctx->index_frame_payload);
+        gnb_send_address_list_through_all_sockets(gnb_core, gnb_core->index_address_ring.address_list, index_worker_ctx->index_frame_payload,1);
     }
 
     node->last_request_addr_sec = index_worker_ctx->now_time_sec;
 
-    GNB_DEBUG5(gnb_core->log, GNB_LOG_ID_INDEX_WORKER, "SEND REQUEST ADDR %u ==>%u lkey[%s] rkey[%s]\n", gnb_core->local_node->uuid32, node->uuid32, GNB_HEX1_BYTE128(gnb_core->local_node->key512), GNB_HEX2_BYTE128(node->key512));
+    GNB_DEBUG5(gnb_core->log, GNB_LOG_ID_INDEX_WORKER, "SEND REQUEST ADDR %llu ==>%llu lkey[%s] rkey[%s]\n", gnb_core->local_node->uuid64, node->uuid64, GNB_HEX1_BYTE128(gnb_core->local_node->key512), GNB_HEX2_BYTE128(node->key512));
  
 }
 
 
 //地址端口探测   gnb_address_t *address
-static void send_detect_addr_frame(gnb_worker_t *gnb_index_worker, gnb_address_t *in_address, uint32_t dst_uuid32){
+static void send_detect_addr_frame(gnb_worker_t *gnb_index_worker, gnb_address_t *in_address, gnb_uuid_t dst_uuid64){
 
     if ( 0 == in_address->port ) {
         return;
@@ -198,18 +184,18 @@ static void send_detect_addr_frame(gnb_worker_t *gnb_index_worker, gnb_address_t
 
     memcpy(detect_addr_frame->data.src_key512, gnb_core->local_node->key512, 64);
 
-    detect_addr_frame->data.src_uuid32 = htonl(gnb_core->local_node->uuid32);
-    detect_addr_frame->data.dst_uuid32 = htonl(dst_uuid32);
+    detect_addr_frame->data.src_uuid64  = gnb_htonll(gnb_core->local_node->uuid64);
+    detect_addr_frame->data.dst_uuid64  = gnb_htonll(dst_uuid64);
     detect_addr_frame->data.src_ts_usec = gnb_htonll(index_worker_ctx->now_time_usec);
 
     //debug_text
-    snprintf(detect_addr_frame->data.text,32,"[%u]DETECT_ADDR[%u]", gnb_core->local_node->uuid32, dst_uuid32);
+    snprintf(detect_addr_frame->data.text,32,"[%llu]DETECT_ADDR[%llu]", gnb_core->local_node->uuid64, dst_uuid64);
 
     if ( 0 == gnb_core->conf->lite_mode ) {
         ed25519_sign(detect_addr_frame->src_sign, (const unsigned char *)&detect_addr_frame->data, sizeof(struct detect_addr_frame_data), gnb_core->ed25519_public_key, gnb_core->ed25519_private_key);
     }
 
-    gnb_send_to_address_through_all_sockets(gnb_core, &address_st, index_worker_ctx->index_frame_payload);
+    gnb_send_to_address_through_all_sockets(gnb_core, &address_st, index_worker_ctx->index_frame_payload, gnb_core->conf->address_detect_interval_usec);
 
     //以下小范围端口探测，只针对ipv4
     if ( AF_INET != address_st.type ) {
@@ -220,7 +206,7 @@ static void send_detect_addr_frame(gnb_worker_t *gnb_index_worker, gnb_address_t
         return;
     }
 
-    GNB_LOG2(gnb_core->log, GNB_LOG_ID_INDEX_WORKER, "send DETECT %u ==> %u address %s port range[%d]\n", gnb_core->local_node->uuid32, dst_uuid32 , GNB_IP_PORT_STR1(&address_st),gnb_core->conf->port_detect_range);
+    GNB_LOG2(gnb_core->log, GNB_LOG_ID_INDEX_WORKER, "send DETECT %llu ==> %llu address %s port range[%d]\n", gnb_core->local_node->uuid64, dst_uuid64 , GNB_IP_PORT_STR1(&address_st),gnb_core->conf->port_detect_range);
 
     uint16_t dst_port = ntohs(address_st.port);
 
@@ -233,9 +219,7 @@ static void send_detect_addr_frame(gnb_worker_t *gnb_index_worker, gnb_address_t
         }
 
         address_st.port = htons(i);
-
-        gnb_send_to_address_through_all_sockets(gnb_core, &address_st, index_worker_ctx->index_frame_payload);
-        //need delay
+        gnb_send_to_address_through_all_sockets(gnb_core, &address_st, index_worker_ctx->index_frame_payload, 1);
 
     }
 
@@ -246,15 +230,14 @@ static void send_detect_addr_frame(gnb_worker_t *gnb_index_worker, gnb_address_t
         }
 
         address_st.port = htons(i);
-        gnb_send_to_address_through_all_sockets(gnb_core, &address_st, index_worker_ctx->index_frame_payload);
-        //need delay
+        gnb_send_to_address_through_all_sockets(gnb_core, &address_st, index_worker_ctx->index_frame_payload, 1);
 
     }
 
 }
 
 
-static void send_detect_addr_frame_arg(gnb_worker_t *gnb_index_worker, gnb_address_t *in_address, uint32_t dst_uuid32, unsigned char agr0){
+static void send_detect_addr_frame_arg(gnb_worker_t *gnb_index_worker, gnb_address_t *in_address, gnb_uuid_t dst_uuid64, unsigned char agr0){
 
     if ( 0 == in_address->port ) {
         return;
@@ -279,12 +262,12 @@ static void send_detect_addr_frame_arg(gnb_worker_t *gnb_index_worker, gnb_addre
 
     memcpy(detect_addr_frame->data.src_key512, gnb_core->local_node->key512, 64);
 
-    detect_addr_frame->data.src_uuid32 = htonl(gnb_core->local_node->uuid32);
-    detect_addr_frame->data.dst_uuid32 = htonl(dst_uuid32);
+    detect_addr_frame->data.src_uuid64  = gnb_htonll(gnb_core->local_node->uuid64);
+    detect_addr_frame->data.dst_uuid64  = gnb_htonll(dst_uuid64);
     detect_addr_frame->data.src_ts_usec = gnb_htonll(index_worker_ctx->now_time_usec);
 
     //debug_text
-    snprintf(detect_addr_frame->data.text,32,"[%u]DETECT_ADDR[%u]", gnb_core->local_node->uuid32, dst_uuid32);
+    snprintf(detect_addr_frame->data.text,32,"[%llu]DETECT_ADDR[%llu]", gnb_core->local_node->uuid64, dst_uuid64);
 
     if ( 0 == gnb_core->conf->lite_mode ) {
         ed25519_sign(detect_addr_frame->src_sign, (const unsigned char *)&detect_addr_frame->data, sizeof(struct detect_addr_frame_data), gnb_core->ed25519_public_key, gnb_core->ed25519_private_key);
@@ -302,27 +285,27 @@ static void detect_node_addr(gnb_worker_t *gnb_index_worker, gnb_node_t *node){
     gnb_address_list_t *resolv_address_list;
     gnb_address_list_t *push_address_list;
 
-    static_address_list = (gnb_address_list_t *)&node->static_address_block;
+    static_address_list  = (gnb_address_list_t *)&node->static_address_block;
     dynamic_address_list = (gnb_address_list_t *)&node->dynamic_address_block;
-    resolv_address_list = (gnb_address_list_t *)&node->resolv_address_block;
-    push_address_list = (gnb_address_list_t *)&node->push_address_block;
+    resolv_address_list  = (gnb_address_list_t *)&node->resolv_address_block;
+    push_address_list    = (gnb_address_list_t *)&node->push_address_block;
 
     int i;
 
     for ( i=0; i<static_address_list->num; i++ ) {
-        send_detect_addr_frame(gnb_index_worker, &static_address_list->array[i],node->uuid32);
+        send_detect_addr_frame(gnb_index_worker, &static_address_list->array[i],node->uuid64);
     }
 
     for ( i=0; i<push_address_list->num; i++ ) {
-        send_detect_addr_frame(gnb_index_worker, &push_address_list->array[i],node->uuid32);
+        send_detect_addr_frame(gnb_index_worker, &push_address_list->array[i],node->uuid64);
     }
 
     for ( i=0; i<dynamic_address_list->num; i++ ) {
-        send_detect_addr_frame(gnb_index_worker, &dynamic_address_list->array[i],node->uuid32);
+        send_detect_addr_frame(gnb_index_worker, &dynamic_address_list->array[i],node->uuid64);
     }
 
     for ( i=0; i<resolv_address_list->num; i++ ) {
-        send_detect_addr_frame(gnb_index_worker, &resolv_address_list->array[i],node->uuid32);
+        send_detect_addr_frame(gnb_index_worker, &resolv_address_list->array[i],node->uuid64);
     }
 
     node->detect_count++;
@@ -340,11 +323,11 @@ static void handle_push_addr_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *i
 
     int i;
 
-    uint32_t nodeid = ntohl(push_addr_frame->data.node_uuid32);
+    gnb_uuid_t nodeid = gnb_ntohll(push_addr_frame->data.node_uuid64);
 
     gnb_node_t *node;
 
-    node = GNB_HASH32_UINT32_GET_PTR(gnb_core->uuid_node_map, nodeid);
+    node = GNB_HASH32_UINT64_GET_PTR(gnb_core->uuid_node_map, nodeid);
 
     if ( NULL == node ) {
         return;
@@ -354,7 +337,7 @@ static void handle_push_addr_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *i
         return;
     }
 
-    if ( gnb_core->local_node->uuid32 == node->uuid32 ) {
+    if ( gnb_core->local_node->uuid64 == node->uuid64 ) {
         return;
     }
 
@@ -363,15 +346,6 @@ static void handle_push_addr_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *i
         return;
     }
 
-#if 0
-    if ( 0 == gnb_core->conf->lite_mode && 0 != ed25519_verify(push_addr_frame->src_sign, (void *)&push_addr_frame->data, sizeof(struct push_addr_frame_data), node->public_key) ) {
-        return;
-    }
-#endif
-
-    node->last_push_addr_sec = index_worker_ctx->now_time_sec;
-    node->detect_count = 0;
-
     push_address_list   = (gnb_address_list_t *)&node->push_address_block;
     detect_address_list = (gnb_address_list_t *)&node->detect_address4_block;
 
@@ -417,8 +391,8 @@ static void handle_push_addr_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *i
     }
 
     //just for log
-    for( i=0; i<dst_address6_list->num; i++ ) {
-        GNB_LOG2(gnb_core->log,GNB_LOG_ID_INDEX_WORKER,"RECEIVE_PUSH_ADDR node=%d %s text='%.*s' action=%c\n", nodeid, GNB_IP_PORT_STR1(&dst_address6_list->array[i]), 32, push_addr_frame->data.text, push_addr_frame->data.arg0);
+    for ( i=0; i<dst_address6_list->num; i++ ) {
+        GNB_LOG2(gnb_core->log,GNB_LOG_ID_INDEX_WORKER,"RECEIVE_PUSH_ADDR node=%llu %s text='%.*s' action=%c\n", nodeid, GNB_IP_PORT_STR1(&dst_address6_list->array[i]), 32, push_addr_frame->data.text, push_addr_frame->data.arg0);
     }
 
     if ( PUSH_ADDR_ACTION_CONNECT == push_addr_frame->data.arg0 ) {
@@ -460,16 +434,23 @@ static void handle_push_addr_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *i
 
     //just for log
     for ( i=0; i<dst_address4_list->num; i++ ) {
-        GNB_LOG2(gnb_core->log,GNB_LOG_ID_INDEX_WORKER,"RECEIVE_PUSH_ADDR node=%d %s text='%.*s' action=%c\n", nodeid, GNB_IP_PORT_STR1(&dst_address4_list->array[i]), 32, push_addr_frame->data.text, push_addr_frame->data.arg0);
+        GNB_LOG2(gnb_core->log,GNB_LOG_ID_INDEX_WORKER,"RECEIVE_PUSH_ADDR node=%llu %s text='%.*s' action=%c\n", nodeid, GNB_IP_PORT_STR1(&dst_address4_list->array[i]), 32, push_addr_frame->data.text, push_addr_frame->data.arg0);
     }
 
+    if ( dst_address6_list->num == 0 && dst_address4_list->num == 0 ) {
+        GNB_LOG2(gnb_core->log,GNB_LOG_ID_INDEX_WORKER,"RECEIVE_PUSH_ADDR ERROR push address incorrect node=%llu text='%.*s' action=%c\n", nodeid,  32, push_addr_frame->data.text, push_addr_frame->data.arg0);
+        return;
+    }
+
+    node->last_push_addr_sec = index_worker_ctx->now_time_sec;
+    node->detect_count = 0;
+
     if ( PUSH_ADDR_ACTION_CONNECT == push_addr_frame->data.arg0 ) {
         for ( i=0; i<dst_address4_list->num; i++ ) {
             send_detect_addr_frame(gnb_core->index_worker, &dst_address4_list->array[i], nodeid);
         }
     }
 
-
 }
 
 
@@ -493,7 +474,7 @@ static void sync_index_node(gnb_worker_t *gnb_index_worker){
 
         node = &gnb_core->ctl_block->node_zone->node[i];
 
-        if ( gnb_core->local_node->uuid32 == node->uuid32 ) {
+        if ( gnb_core->local_node->uuid64 == node->uuid64 ) {
             continue;
         }
 
@@ -537,9 +518,9 @@ static void handle_echo_addr_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *i
 
     echo_addr_frame_t *echo_addr_frame = (echo_addr_frame_t *)&index_worker_in_data->payload_st.data;
 
-    uint32_t dst_uuid32 = ntohl(echo_addr_frame->data.dst_uuid32);
+    gnb_uuid_t dst_uuid64 = gnb_ntohll(echo_addr_frame->data.dst_uuid64);
 
-    if ( dst_uuid32 != gnb_core->local_node->uuid32 ) {
+    if ( dst_uuid64 != gnb_core->local_node->uuid64 ) {
         return;
     }
 
@@ -553,6 +534,9 @@ static void handle_echo_addr_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *i
         gnb_set_address4(address, &sockaddress->addr.in);
     }
 
+    /*
+    如果 index 节点有多个 ipv6 地址,这里得到地址不一定与发送的目的地址相同
+    */
     int idx = gnb_address_list_find(gnb_core->index_address_ring.address_list, address);
 
     if ( -1 == idx ) {
@@ -568,13 +552,13 @@ static void handle_echo_addr_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *i
     if ( '6' == echo_addr_frame->data.addr_type ) {
         memcpy(&gnb_core->local_node->udp_sockaddr6.sin6_addr, &echo_addr_frame->data.addr, 16);
         gnb_core->local_node->udp_sockaddr6.sin6_port = echo_addr_frame->data.port;
-        GNB_LOG3(gnb_core->log,GNB_LOG_ID_INDEX_WORKER,"get echo address %s:%d from index %s:%d\n", GNB_ADDR6STR1(echo_addr_frame->data.addr), ntohs(echo_addr_frame->data.port), GNB_SOCKETADDRSTR2(sockaddress), ntohs(sockaddress->addr.in6.sin6_port));
+        GNB_LOG3(gnb_core->log,GNB_LOG_ID_INDEX_WORKER,"get echo address %s:%d from index %s\n", GNB_ADDR6STR1(echo_addr_frame->data.addr), ntohs(echo_addr_frame->data.port), GNB_SOCKETADDRSTR2(sockaddress));
     } else if ( '4' == echo_addr_frame->data.addr_type ) {
         memcpy(&gnb_core->local_node->udp_sockaddr4.sin_addr, &echo_addr_frame->data.addr, 4);
         gnb_core->local_node->udp_sockaddr4.sin_port = echo_addr_frame->data.port;
-        GNB_LOG3(gnb_core->log,GNB_LOG_ID_INDEX_WORKER,"get echo address %s:%d from index %s:%d\n", GNB_ADDR4STR1(echo_addr_frame->data.addr), ntohs(echo_addr_frame->data.port), GNB_SOCKETADDRSTR2(sockaddress), ntohs(sockaddress->addr.in.sin_port));
+        GNB_LOG3(gnb_core->log,GNB_LOG_ID_INDEX_WORKER,"get echo address %s:%d from index %s\n", GNB_ADDR4STR1(echo_addr_frame->data.addr), ntohs(echo_addr_frame->data.port), GNB_SOCKETADDRSTR2(sockaddress));
     } else {
-        GNB_LOG3(gnb_core->log,GNB_LOG_ID_INDEX_WORKER,"handle echo address type error%.*s from %s:%d\n", 80, echo_addr_frame->data.text, GNB_SOCKETADDRSTR2(sockaddress), ntohs(sockaddress->addr.in.sin_port));
+        GNB_LOG3(gnb_core->log,GNB_LOG_ID_INDEX_WORKER,"handle echo address type error%.*s from %s\n", 80, echo_addr_frame->data.text, GNB_SOCKETADDRSTR2(sockaddress));
         return;
     }
 
@@ -589,14 +573,14 @@ static void handle_detect_addr_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t
 
     detect_addr_frame_t *detect_addr_frame = (detect_addr_frame_t *)&index_worker_in_data->payload_st.data;
 
-    uint32_t src_uuid32 = ntohl(detect_addr_frame->data.src_uuid32);
-    uint32_t dst_uuid32 = ntohl(detect_addr_frame->data.dst_uuid32);
+    gnb_uuid_t src_uuid64 = gnb_ntohll(detect_addr_frame->data.src_uuid64);
+    gnb_uuid_t dst_uuid64 = gnb_ntohll(detect_addr_frame->data.dst_uuid64);
 
     gnb_sockaddress_t *sockaddress = &index_worker_in_data->node_addr_st;
 
     gnb_node_t *src_node;
 
-    src_node = GNB_HASH32_UINT32_GET_PTR(gnb_core->uuid_node_map, src_uuid32);
+    src_node = GNB_HASH32_UINT64_GET_PTR(gnb_core->uuid_node_map, src_uuid64);
 
     if ( NULL==src_node ) {
         return;
@@ -614,12 +598,12 @@ static void handle_detect_addr_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t
     dynamic_address_list = (gnb_address_list_t *)&src_node->dynamic_address_block;
 
     if ( 0 != memcmp(src_node->key512, detect_addr_frame->data.src_key512, 64) ) {
-        GNB_LOG2(gnb_core->log,GNB_LOG_ID_INDEX_WORKER,"handle detect addr invalid key512 src=%u %s ！！\n", src_uuid32, GNB_SOCKETADDRSTR1(sockaddress));
+        GNB_LOG2(gnb_core->log,GNB_LOG_ID_INDEX_WORKER,"handle detect addr invalid key512 src=%llu %s ！！\n", src_uuid64, GNB_SOCKETADDRSTR1(sockaddress));
         return;
     }
 
     if ( 0 == gnb_core->conf->lite_mode && !ed25519_verify(detect_addr_frame->src_sign, (const unsigned char *)&detect_addr_frame->data, sizeof(struct detect_addr_frame_data), src_node->public_key) ) {
-        GNB_LOG2(gnb_core->log,GNB_LOG_ID_INDEX_WORKER,"handle detect addr src=%u invalid signature %s !!\n", src_uuid32, GNB_SOCKETADDRSTR1(sockaddress));
+        GNB_LOG2(gnb_core->log,GNB_LOG_ID_INDEX_WORKER,"handle detect addr src=%llu invalid signature %s !!\n", src_uuid64, GNB_SOCKETADDRSTR1(sockaddress));
         return;
     }
 
@@ -630,7 +614,19 @@ static void handle_detect_addr_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t
     address->socket_idx = index_worker_in_data->socket_idx;
     address->ts_sec = index_worker_ctx->now_time_usec;
 
-    if (AF_INET6 == sockaddress->addr_type){
+    if ( AF_INET6 == sockaddress->addr_type ) {
+
+        if ( 0 != gnb_determine_subnet6_prefixlen96(sockaddress->addr.in6.sin6_addr, gnb_core->local_node->tun_ipv6_addr ) ) {
+
+            GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "handle_detect_addr_frame IPV6 Warning src[%u]->dst[%u] idx=%u %s\n",
+                    src_node->uuid64, dst_uuid64,
+                    index_worker_in_data->socket_idx,
+                    GNB_ADDR6STR1(&sockaddress->addr.in6.sin6_addr)
+            );
+
+            return;
+
+        }
 
         if ( 'e' == detect_addr_frame->data.arg0 ) {
             src_node->udp_addr_status |= GNB_NODE_STATUS_IPV6_PONG;
@@ -647,11 +643,23 @@ static void handle_detect_addr_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t
 
         gnb_set_address6(address, &sockaddress->addr.in6);
 
-        GNB_LOG2(gnb_core->log, GNB_LOG_ID_INDEX_WORKER, "==###== RECEIVE_DETECT_ADDR6 node[%u]->[%u] idx[%u]%s[%c] ==###==\n", src_uuid32, dst_uuid32, src_node->socket6_idx, GNB_IP_PORT_STR1(address), detect_addr_frame->data.arg0);
+        GNB_LOG2(gnb_core->log, GNB_LOG_ID_INDEX_WORKER, "==###== RECEIVE_DETECT_ADDR6 node[%llu]->[%llu] idx[%u]%s[%c] ==###==\n", src_uuid64, dst_uuid64, src_node->socket6_idx, GNB_IP_PORT_STR1(address), detect_addr_frame->data.arg0);
     }
 
     if ( AF_INET == sockaddress->addr_type ) {
 
+        if ( 0 != gnb_determine_subnet4(sockaddress->addr.in.sin_addr,  gnb_core->local_node->tun_addr4, gnb_core->local_node->tun_netmask_addr4) ) {
+
+            GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "handle_detect_addr_frame IPV4 Warning src[%llu]->dst[%llu] idx=%u %s\n",
+                    src_node->uuid64, dst_uuid64,
+                    index_worker_in_data->socket_idx,
+                    GNB_ADDR4STR1(&sockaddress->addr.in.sin_addr)
+            );
+            
+            return;
+            
+        }
+
         if ( 'e' == detect_addr_frame->data.arg0 ) {
             src_node->udp_addr_status |= GNB_NODE_STATUS_IPV4_PONG;
             src_node->addr4_update_ts_sec = index_worker_ctx->now_time_sec;
@@ -667,7 +675,7 @@ static void handle_detect_addr_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t
 
         gnb_set_address4(address, &sockaddress->addr.in);
 
-        GNB_LOG2(gnb_core->log, GNB_LOG_ID_INDEX_WORKER, "==###== RECEIVE_DETECT_ADDR4 node[%u]->[%u] idx[%u]%s[%c] ==###==\n", src_uuid32, dst_uuid32, src_node->socket4_idx, GNB_IP_PORT_STR1(address), detect_addr_frame->data.arg0);
+        GNB_LOG2(gnb_core->log, GNB_LOG_ID_INDEX_WORKER, "==###== RECEIVE_DETECT_ADDR4 node[%llu]->[%llu] idx[%u]%s[%c] ==###==\n", src_uuid64, dst_uuid64, src_node->socket4_idx, GNB_IP_PORT_STR1(address), detect_addr_frame->data.arg0);
 
     }
 
@@ -676,8 +684,8 @@ static void handle_detect_addr_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t
     gnb_address_list_update(dynamic_address_list, address);
 
     if ( 'e' != detect_addr_frame->data.arg0 ) {
-        send_detect_addr_frame_arg(gnb_core->index_worker, address,  src_uuid32, 'e');
-        GNB_LOG3(gnb_core->log, GNB_LOG_ID_INDEX_WORKER, "ECHO DETECT -> node[%u] idx[%u]%s\n", src_uuid32, src_node->socket4_idx, GNB_IP_PORT_STR1(address));
+        send_detect_addr_frame_arg(gnb_core->index_worker, address,  src_uuid64, 'e');
+        GNB_LOG3(gnb_core->log, GNB_LOG_ID_INDEX_WORKER, "ECHO DETECT -> node[%llu] idx[%u]%s\n", src_uuid64, src_node->socket4_idx, GNB_IP_PORT_STR1(address));
     }
 
 }
@@ -764,7 +772,7 @@ static void* thread_worker_func( void *data ) {
 
         handle_recv_queue(gnb_core);
 
-        if ( 0 ==gnb_core->index_address_ring.address_list->num ) {
+        if ( 0 == gnb_core->index_address_ring.address_list->num ) {
             goto next;
         }
 
@@ -795,7 +803,7 @@ static void init(gnb_worker_t *gnb_worker, void *ctx){
     index_worker_ctx_t *index_worker_ctx = (index_worker_ctx_t *)gnb_heap_alloc(gnb_core->heap, sizeof(index_worker_ctx_t));
     memset(index_worker_ctx, 0, sizeof(index_worker_ctx_t));
 
-    index_worker_ctx->index_frame_payload = (gnb_payload16_t *)gnb_heap_alloc(gnb_core->heap,GNB_MAX_PAYLOAD_SIZE);
+    index_worker_ctx->index_frame_payload = (gnb_payload16_t *)gnb_heap_alloc(gnb_core->heap, gnb_core->conf->payload_block_size);
     index_worker_ctx->index_frame_payload->type = GNB_PAYLOAD_TYPE_INDEX;
     index_worker_ctx->gnb_core = gnb_core;
 
@@ -805,6 +813,7 @@ static void init(gnb_worker_t *gnb_worker, void *ctx){
     gnb_worker->ring_buffer_out = NULL;
     gnb_worker->ctx = index_worker_ctx;
 
+    GNB_LOG1(gnb_core->log, GNB_LOG_ID_INDEX_WORKER, "%s in ring buffer size = %d\n", gnb_worker->name, gnb_core->conf->index_woker_queue_length);
     GNB_LOG1(gnb_core->log, GNB_LOG_ID_INDEX_WORKER, "%s init finish\n", gnb_worker->name);
 
 }
