@@ -41,23 +41,25 @@
 #include "gnb_keys.h"
 #include "gnb_time.h"
 #include "gnb_udp.h"
+#include "gnb_hash32.h"
 #include "ed25519/ed25519.h"
 #include "ed25519/sha512.h"
+#include "gnb_unified_forwarding.h"
 #include "gnb_binary.h"
 
 
-gnb_node_t * gnb_node_init(gnb_core_t *gnb_core, uint32_t uuid32){
+gnb_node_t * gnb_node_init(gnb_core_t *gnb_core, gnb_uuid_t uuid64){
 
     gnb_node_t *node = &gnb_core->ctl_block->node_zone->node[gnb_core->node_nums];
 
     memset(node,0,sizeof(gnb_node_t));
 
-    node->uuid32 = uuid32;
+    node->uuid64 = uuid64;
 
     node->type =  GNB_NODE_TYPE_STD;
 
-    uint32_t node_id_network_order;
-    uint32_t local_node_id_network_order;
+    gnb_uuid_t node_id_network_order;
+    gnb_uuid_t local_node_id_network_order;
 
     gnb_address_list_t *static_address_list;
     gnb_address_list_t *dynamic_address_list;
@@ -81,9 +83,9 @@ gnb_node_t * gnb_node_init(gnb_core_t *gnb_core, uint32_t uuid32){
 
     if ( 0 == gnb_core->conf->lite_mode ) {
 
-        if ( gnb_core->conf->local_uuid != uuid32 ) {
+        if ( gnb_core->conf->local_uuid != uuid64 ) {
 
-            gnb_load_public_key(gnb_core, uuid32, node->public_key);
+            gnb_load_public_key(gnb_core, uuid64, node->public_key);
             ed25519_key_exchange(node->shared_secret, node->public_key, gnb_core->ed25519_private_key);
 
         } else {
@@ -97,15 +99,15 @@ gnb_node_t * gnb_node_init(gnb_core_t *gnb_core, uint32_t uuid32){
     } else {
 
         //lite mode
-        if ( gnb_core->conf->local_uuid == uuid32 ) {
+        if ( gnb_core->conf->local_uuid == uuid64 ) {
             memset(gnb_core->ed25519_private_key, 0, 64);
             memset(gnb_core->ed25519_public_key,  0,32);
         }
 
         memset(node->public_key, 0, 32);
 
-        node_id_network_order       = htonl(uuid32);
-        local_node_id_network_order = htonl(gnb_core->conf->local_uuid);
+        node_id_network_order       = gnb_htonll(uuid64);
+        local_node_id_network_order = gnb_htonll(gnb_core->conf->local_uuid);
 
         memcpy(node->public_key, &node_id_network_order, 4);
 
@@ -137,7 +139,7 @@ void gnb_init_node_key512(gnb_core_t *gnb_core){
 
     int i;
 
-    uint32_t uuid32;
+    gnb_uuid_t uuid64;
 
     gnb_node_t *node;
 
@@ -161,11 +163,11 @@ void gnb_init_node_key512(gnb_core_t *gnb_core){
 }
 
 
-void gnb_add_forward_node_ring(gnb_core_t *gnb_core, uint32_t uuid32){
+void gnb_add_forward_node_ring(gnb_core_t *gnb_core, gnb_uuid_t uuid64){
 
     gnb_node_t *node;
 
-    node = GNB_HASH32_UINT32_GET_PTR(gnb_core->uuid_node_map, uuid32);
+    node = GNB_HASH32_UINT64_GET_PTR(gnb_core->uuid_node_map, uuid64);
 
     if ( NULL == node ) {
         return;
@@ -187,6 +189,32 @@ void gnb_add_forward_node_ring(gnb_core_t *gnb_core, uint32_t uuid32){
 }
 
 
+void gnb_add_index_node_ring(gnb_core_t *gnb_core, gnb_uuid_t uuid64){
+
+    gnb_node_t *node;
+
+    node = GNB_HASH32_UINT64_GET_PTR(gnb_core->uuid_node_map, uuid64);
+
+    if ( NULL == node ) {
+        return;
+    }
+
+    if ( gnb_core->index_node_ring.num >= GNB_MAX_NODE_RING ) {
+        gnb_core->index_node_ring.num = GNB_MAX_NODE_RING;
+        return;
+    }
+
+    node->type |= GNB_NODE_TYPE_IDX;
+
+    gnb_core->index_node_ring.nodes[gnb_core->index_node_ring.num] = node;
+
+    gnb_core->index_node_ring.cur_index = gnb_core->index_node_ring.num;
+
+    gnb_core->index_node_ring.num++;
+
+}
+
+
 gnb_node_t* gnb_select_forward_node(gnb_core_t *gnb_core){
 
     int i;
@@ -351,18 +379,18 @@ void gnb_send_available_address_list(gnb_core_t *gnb_core, gnb_address_list_t *a
 }
 
 
-void gnb_send_address_list_through_all_sockets(gnb_core_t *gnb_core, gnb_address_list_t *address_list, gnb_payload16_t *payload){
+void gnb_send_address_list_through_all_sockets(gnb_core_t *gnb_core, gnb_address_list_t *address_list, gnb_payload16_t *payload, uint32_t interval_usec){
 
     int i;
 
     for ( i=0; i<address_list->num; i++ ) {
-         gnb_send_to_address_through_all_sockets(gnb_core, &address_list->array[i], payload);
+         gnb_send_to_address_through_all_sockets(gnb_core, &address_list->array[i], payload, interval_usec);
     }
 
 }
 
 
-void gnb_send_to_address_through_all_sockets(gnb_core_t *gnb_core, gnb_address_t *address, gnb_payload16_t *payload){
+void gnb_send_to_address_through_all_sockets(gnb_core_t *gnb_core, gnb_address_t *address, gnb_payload16_t *payload, uint32_t interval_usec){
 
     struct sockaddr_in  in;
     struct sockaddr_in6 in6;
@@ -384,6 +412,7 @@ void gnb_send_to_address_through_all_sockets(gnb_core_t *gnb_core, gnb_address_t
 
         for (i=0; i<gnb_core->conf->udp6_socket_num; i++) {
             sendto(gnb_core->udp_ipv6_sockets[i], (void *)payload, GNB_PAYLOAD16_FRAME_SIZE(payload), 0, (struct sockaddr *)&in6, sizeof(struct sockaddr_in6) );
+            GNB_SLEEP_MILLISECOND( interval_usec/1000 );
         }
 
     }
@@ -398,6 +427,7 @@ void gnb_send_to_address_through_all_sockets(gnb_core_t *gnb_core, gnb_address_t
 
         for (i=0; i<gnb_core->conf->udp4_socket_num; i++) {
             sendto(gnb_core->udp_ipv4_sockets[i], (void *)payload, GNB_PAYLOAD16_FRAME_SIZE(payload), 0, (struct sockaddr *)&in, sizeof(struct sockaddr_in));
+            GNB_SLEEP_MILLISECOND( interval_usec/1000 );
         }
 
     }
@@ -472,6 +502,64 @@ SIMPLE_LOAD_BALANCE:
 }
 
 
+gnb_node_t* gnb_select_index_nodes(gnb_core_t *gnb_core){
+
+    int i;
+
+    gnb_node_t *gnb_index_node;
+
+    if ( 0 == gnb_core->index_node_ring.num ) {
+        return NULL;
+    }
+
+    if ( 1 == gnb_core->index_node_ring.num ) {
+        return gnb_core->index_node_ring.nodes[0];
+    }
+
+    if ( GNB_MULTI_ADDRESS_TYPE_SIMPLE_FAULT_TOLERANT == gnb_core->conf->multi_index_type ) {
+        goto SIMPLE_FAULT_TOLERANT;
+    }
+
+    if ( GNB_MULTI_ADDRESS_TYPE_SIMPLE_LOAD_BALANCE == gnb_core->conf->multi_index_type ) {
+        goto SIMPLE_LOAD_BALANCE;
+    }
+
+SIMPLE_FAULT_TOLERANT:
+
+    for ( i=0; i<gnb_core->index_node_ring.num; i++ ) {
+
+        gnb_index_node = gnb_core->index_node_ring.nodes[i];
+
+        if ( GNB_NODE_STATUS_IPV6_PONG & gnb_index_node->udp_addr_status || GNB_NODE_STATUS_IPV4_PONG  & gnb_index_node->udp_addr_status ) {
+            return gnb_core->index_node_ring.nodes[i];
+        }
+
+    }
+
+    return gnb_core->index_node_ring.nodes[0];
+
+SIMPLE_LOAD_BALANCE:
+
+    for ( i=0; i<gnb_core->index_node_ring.num; i++ ) {
+
+        gnb_index_node = gnb_core->index_node_ring.nodes[gnb_core->index_node_ring.cur_index];
+
+        gnb_core->index_node_ring.cur_index++;
+
+        if ( gnb_core->index_node_ring.cur_index >= gnb_core->index_node_ring.num ) {
+            gnb_core->index_node_ring.cur_index = 0;
+        }
+
+        if ( GNB_NODE_STATUS_IPV6_PONG & gnb_index_node->udp_addr_status || GNB_NODE_STATUS_IPV4_PONG  & gnb_index_node->udp_addr_status ) {
+            return gnb_core->index_node_ring.nodes[i];
+        }
+    }
+
+    return gnb_core->index_node_ring.nodes[0];
+
+}
+
+
 gnb_address_t* gnb_select_available_address4(gnb_core_t *gnb_core, gnb_node_t *node){
 
     gnb_address_t *address = NULL;
@@ -629,10 +717,11 @@ finish:
 }
 
 
-int gnb_forward_payload_to_node(gnb_core_t *gnb_core, gnb_node_t *node, gnb_payload16_t *payload){
+int gnb_p2p_forward_payload_to_node(gnb_core_t *gnb_core, gnb_node_t *node, gnb_payload16_t *payload){
 
     int ret;
 
+    // gnb_core->conf->udp_socket_type 默认是 GNB_ADDR_TYPE_IPV4 | GNB_ADDR_TYPE_IPV6;
     if ( GNB_ADDR_TYPE_IPV4 == gnb_core->conf->udp_socket_type ) {
         goto send_by_ipv4;
     } else if ( GNB_ADDR_TYPE_IPV6 == gnb_core->conf->udp_socket_type ) {
@@ -676,3 +765,195 @@ finish:
     return 0;
 
 }
+
+
+#define GNB_SEND_BY_UNSET    (0x0)
+#define GNB_SEND_BY_P2P_IPV6 (0x1)
+#define GNB_SEND_BY_P2P_IPV4 (0x2)
+#define GNB_SEND_BY_FWD_IPV6 (0x3)
+#define GNB_SEND_BY_FWD_IPV4 (0x4)
+void gnb_std_uf_forward_payload_to_node(gnb_core_t *gnb_core, gnb_node_t *node, gnb_payload16_t *payload){
+
+    int ret;
+    int send_flag = GNB_SEND_BY_UNSET;
+    gnb_node_t *fwd_node;
+
+    if ( (node->udp_addr_status & GNB_NODE_STATUS_IPV6_PONG) && (node->udp_addr_status & GNB_NODE_STATUS_IPV4_PONG) ) {
+
+        if ( 0 == node->addr4_ping_latency_usec ) {
+            send_flag = GNB_SEND_BY_P2P_IPV6;
+            goto p2p_forwarding;
+        }
+
+        if ( 0 == node->addr6_ping_latency_usec ) {
+            send_flag = GNB_SEND_BY_P2P_IPV4;
+            goto p2p_forwarding;
+        }
+
+        if ( node->addr4_ping_latency_usec >= node->addr6_ping_latency_usec ) {
+            send_flag = GNB_SEND_BY_P2P_IPV6;
+            goto p2p_forwarding;
+        } else {
+            send_flag = GNB_SEND_BY_P2P_IPV4;
+            goto p2p_forwarding;
+        }            
+
+    } else if ( (node->udp_addr_status & GNB_NODE_STATUS_IPV6_PONG) && (GNB_ADDR_TYPE_IPV6 & gnb_core->conf->udp_socket_type) ) {
+        send_flag = GNB_SEND_BY_P2P_IPV6;
+    } else if ( (node->udp_addr_status & GNB_NODE_STATUS_IPV4_PONG) && (GNB_ADDR_TYPE_IPV4 & gnb_core->conf->udp_socket_type) ) {
+        send_flag = GNB_SEND_BY_P2P_IPV4;
+    } else {
+        send_flag = GNB_SEND_BY_UNSET;
+        goto try_to_standard_forwarding;
+    }
+
+p2p_forwarding:
+
+    if ( GNB_SEND_BY_P2P_IPV6 == send_flag ) {
+
+        if ( memcmp(&node->udp_sockaddr6.sin6_addr, &in6addr_any,sizeof(struct in6_addr)) ) {
+            return;
+        }
+
+        sendto(gnb_core->udp_ipv6_sockets[node->socket6_idx],(void *)payload, GNB_PAYLOAD16_FRAME_SIZE(payload), 0, (struct sockaddr *)&node->udp_sockaddr6, sizeof(struct sockaddr_in6) );
+        return;
+
+    } else if ( GNB_SEND_BY_P2P_IPV4 == send_flag ) {
+
+        if ( INADDR_ANY == node->udp_sockaddr4.sin_addr.s_addr ) {
+            return;
+        }
+
+        sendto(gnb_core->udp_ipv4_sockets[node->socket4_idx], (void *)payload, GNB_PAYLOAD16_FRAME_SIZE(payload), 0, (struct sockaddr *)&node->udp_sockaddr4, sizeof(struct sockaddr_in));
+        return;
+
+    }
+
+try_to_standard_forwarding:
+
+    //如果无法 point to point forwarding 就尝试通过 standard forwarding 节点转发
+    fwd_node = gnb_select_forward_node(gnb_core);
+
+    if ( NULL == fwd_node ) {
+        goto try_to_unified_forwarding;
+    }
+
+    if ( (fwd_node->udp_addr_status & GNB_NODE_STATUS_IPV6_PONG) && (fwd_node->udp_addr_status & GNB_NODE_STATUS_IPV4_PONG) ) {
+
+        if ( 0 == fwd_node->addr4_ping_latency_usec ) {
+            send_flag = GNB_SEND_BY_FWD_IPV6;
+            goto standard_forwarding;
+        }
+
+        if ( 0 == fwd_node->addr6_ping_latency_usec ) {
+            send_flag = GNB_SEND_BY_FWD_IPV4;
+            goto standard_forwarding;
+        }
+
+        if ( fwd_node->addr4_ping_latency_usec >= fwd_node->addr6_ping_latency_usec ) {
+            send_flag = GNB_SEND_BY_FWD_IPV6;
+            goto standard_forwarding;
+        } else {
+            send_flag = GNB_SEND_BY_FWD_IPV4;
+            goto standard_forwarding;
+        }            
+
+    } else if ( (fwd_node->udp_addr_status & GNB_NODE_STATUS_IPV6_PONG) && (GNB_ADDR_TYPE_IPV6 & gnb_core->conf->udp_socket_type) ) {
+        send_flag = GNB_SEND_BY_FWD_IPV6;
+    } else if ( (fwd_node->udp_addr_status & GNB_NODE_STATUS_IPV4_PONG) && (GNB_ADDR_TYPE_IPV4 & gnb_core->conf->udp_socket_type) ) {
+        send_flag = GNB_SEND_BY_FWD_IPV4;
+    } else {
+        send_flag = GNB_SEND_BY_UNSET;
+        goto try_to_unified_forwarding;
+    }
+
+standard_forwarding:
+
+    if ( GNB_SEND_BY_FWD_IPV6 == send_flag ) {
+
+        if ( memcmp(&fwd_node->udp_sockaddr6.sin6_addr, &in6addr_any,sizeof(struct in6_addr)) ) {
+            return;
+        }
+
+        sendto(gnb_core->udp_ipv6_sockets[fwd_node->socket6_idx],(void *)payload, GNB_PAYLOAD16_FRAME_SIZE(payload), 0, (struct sockaddr *)&fwd_node->udp_sockaddr6, sizeof(struct sockaddr_in6) );
+        return;
+
+    } else if ( GNB_SEND_BY_FWD_IPV4 == send_flag ) {
+
+        if ( INADDR_ANY == fwd_node->udp_sockaddr4.sin_addr.s_addr ) {
+            return;
+        }
+
+        sendto(gnb_core->udp_ipv4_sockets[fwd_node->socket4_idx], (void *)payload, GNB_PAYLOAD16_FRAME_SIZE(payload), 0, (struct sockaddr *)&fwd_node->udp_sockaddr4, sizeof(struct sockaddr_in));
+        return;
+
+    }
+
+try_to_unified_forwarding:
+    //如果无法 standard forwarding 就尝试通过 unified forwarding 节点转发
+    gnb_setup_unified_forwarding_nodeid(gnb_core, node);
+
+    if ( 0 == node->unified_forwarding_nodeid ) {
+        return;
+    }
+
+    fwd_node = GNB_HASH32_UINT64_GET_PTR(gnb_core->uuid_node_map, node->unified_forwarding_nodeid);
+
+    if ( NULL == fwd_node ) {
+        return;
+    }
+
+    //如果对端的fwd_node 关闭了 std forwarding, payload 转过去后会被 drop 掉
+    if ( (fwd_node->udp_addr_status & GNB_NODE_STATUS_IPV6_PONG) && (fwd_node->udp_addr_status & GNB_NODE_STATUS_IPV4_PONG) ) {
+
+        if ( 0 == fwd_node->addr4_ping_latency_usec ) {
+            send_flag = GNB_SEND_BY_FWD_IPV6;
+            goto unified_forwarding;
+        }
+
+        if ( 0 == fwd_node->addr6_ping_latency_usec ) {
+            send_flag = GNB_SEND_BY_FWD_IPV4;
+            goto unified_forwarding;
+        }
+
+        if ( fwd_node->addr4_ping_latency_usec >= fwd_node->addr6_ping_latency_usec ) {
+            send_flag = GNB_SEND_BY_FWD_IPV6;
+            goto unified_forwarding;
+        } else {
+            send_flag = GNB_SEND_BY_FWD_IPV4;
+            goto unified_forwarding;
+        }            
+
+    } else if ( (fwd_node->udp_addr_status & GNB_NODE_STATUS_IPV6_PONG) && (GNB_ADDR_TYPE_IPV6 & gnb_core->conf->udp_socket_type) ) {
+        send_flag = GNB_SEND_BY_FWD_IPV6;
+    } else if ( (fwd_node->udp_addr_status & GNB_NODE_STATUS_IPV4_PONG) && (GNB_ADDR_TYPE_IPV4 & gnb_core->conf->udp_socket_type) ) {
+        send_flag = GNB_SEND_BY_FWD_IPV4;
+    } else {
+        send_flag = GNB_SEND_BY_UNSET;
+    }
+
+unified_forwarding:
+
+    if ( GNB_SEND_BY_FWD_IPV6 == send_flag ) {
+
+        if ( memcmp(&fwd_node->udp_sockaddr6.sin6_addr,&in6addr_any,sizeof(struct in6_addr)) ) {
+            return;
+        }
+
+        sendto(gnb_core->udp_ipv6_sockets[fwd_node->socket6_idx],(void *)payload, GNB_PAYLOAD16_FRAME_SIZE(payload), 0, (struct sockaddr *)&fwd_node->udp_sockaddr6, sizeof(struct sockaddr_in6) );
+        return;
+
+    } else if ( GNB_SEND_BY_FWD_IPV4 == send_flag ) {
+
+        if ( INADDR_ANY == fwd_node->udp_sockaddr4.sin_addr.s_addr ) {
+            return;
+        }
+
+        sendto(gnb_core->udp_ipv4_sockets[fwd_node->socket4_idx], (void *)payload, GNB_PAYLOAD16_FRAME_SIZE(payload), 0, (struct sockaddr *)&fwd_node->udp_sockaddr4, sizeof(struct sockaddr_in));
+        return;
+
+    }
+
+    return;
+
+}
