@@ -44,17 +44,13 @@
 
 //节点同步检测的时间间隔
 #define GNB_NODE_SYNC_INTERVAL_TIME_SEC   10
-
 //对一个node的ping时间间隔
 #define GNB_NODE_PING_INTERVAL_SEC        37
-
 //对于一个节点必须收到的 ping 或 pong 的时间间隔
 #define GNB_NODE_UPDATE_INTERVAL_SEC      (GNB_NODE_PING_INTERVAL_SEC * 2 + 1)
 
-
 #define GNB_UF_NODES_NOTIFY_INTERVAL_SEC  35
 
-
 typedef struct _node_worker_ctx_t {
 
     gnb_core_t *gnb_core;
@@ -71,6 +67,9 @@ typedef struct _node_worker_ctx_t {
 }node_worker_ctx_t;
 
 
+void update_node_crypto_key(gnb_core_t *gnb_core, uint64_t now_sec);
+
+
 static void update_unified_forwarding_node_array(gnb_core_t *gnb_core, gnb_node_t *node, gnb_node_t *uf_node) {
 
     int i;
@@ -82,12 +81,12 @@ static void update_unified_forwarding_node_array(gnb_core_t *gnb_core, gnb_node_
 
     for ( i=0; i<GNB_UNIFIED_FORWARDING_NODE_ARRAY_SIZE; i++ ) {
 
-        if ( 0 == node->unified_forwarding_node_array[i].uuid32 ) {
+        if ( 0 == node->unified_forwarding_node_array[i].uuid64 ) {
             find_idx = i;
             break;
         }
 
-        if ( uf_node->uuid32 == node->unified_forwarding_node_array[i].uuid32 ) {
+        if ( uf_node->uuid64 == node->unified_forwarding_node_array[i].uuid64 ) {
             find_idx = i;
             break;
         }
@@ -108,7 +107,7 @@ static void update_unified_forwarding_node_array(gnb_core_t *gnb_core, gnb_node_
         find_idx = pop_idx;
     }
 
-    node->unified_forwarding_node_array[find_idx].uuid32 = uf_node->uuid32;
+    node->unified_forwarding_node_array[find_idx].uuid64 = uf_node->uuid64;
     node->unified_forwarding_node_array[find_idx].last_ts_sec = node_worker_ctx->now_time_sec;
 
 }
@@ -121,37 +120,37 @@ static void handle_uf_node_notify_frame(gnb_core_t *gnb_core, gnb_worker_in_data
 
     uf_node_notify_frame_t *uf_node_notify_frame = (uf_node_notify_frame_t *)&node_worker_in_data->payload_st.data;
 
-    uint32_t src_uuid32    = ntohl(uf_node_notify_frame->data.src_uuid32);
-    uint32_t dst_uuid32    = ntohl(uf_node_notify_frame->data.dst_uuid32);
-    uint32_t df_uuid32     = ntohl(uf_node_notify_frame->data.df_uuid32);
+    gnb_uuid_t src_uuid64    = gnb_ntohll(uf_node_notify_frame->data.src_uuid64);
+    gnb_uuid_t dst_uuid64    = gnb_ntohll(uf_node_notify_frame->data.dst_uuid64);
+    gnb_uuid_t df_uuid64     = gnb_ntohll(uf_node_notify_frame->data.df_uuid64);
 
     gnb_sockaddress_t *node_addr = &node_worker_in_data->node_addr_st;
 
-    if ( src_uuid32 == gnb_core->local_node->uuid32 || dst_uuid32 != gnb_core->local_node->uuid32 || df_uuid32 == gnb_core->local_node->uuid32 ) {
-        GNB_LOG2(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_uf_node_notify_frame error local=%u src=%u dst=%u\n", gnb_core->local_node->uuid32, src_uuid32, dst_uuid32);
+    if ( src_uuid64 == gnb_core->local_node->uuid64 || dst_uuid64 != gnb_core->local_node->uuid64 || df_uuid64 == gnb_core->local_node->uuid64 ) {
+        GNB_LOG2(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_uf_node_notify_frame error local=%llu src=%llu dst=%llu\n", gnb_core->local_node->uuid64, src_uuid64, dst_uuid64);
         return;
     }
 
-    gnb_node_t *src_node = GNB_HASH32_UINT32_GET_PTR(gnb_core->uuid_node_map, src_uuid32);
+    gnb_node_t *src_node = GNB_HASH32_UINT64_GET_PTR(gnb_core->uuid_node_map, src_uuid64);
 
     if ( NULL==src_node ) {
-        GNB_LOG2(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_uf_node_notify_frame src node=%u is miss\n", src_uuid32);
+        GNB_LOG2(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_uf_node_notify_frame src node=%llu is miss\n", src_uuid64);
         return;
     }
 
     if ( 0 == gnb_core->conf->lite_mode && !ed25519_verify(uf_node_notify_frame->src_sign, (const unsigned char *)&uf_node_notify_frame->data, sizeof(struct uf_node_notify_frame_data), src_node->public_key) ) {
-        GNB_LOG2(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_uf_node_notify_frame invalid signature src=%u %s\n", src_uuid32, GNB_SOCKETADDRSTR1(node_addr));
+        GNB_LOG2(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_uf_node_notify_frame invalid signature src=%llu %s\n", src_uuid64, GNB_SOCKETADDRSTR1(node_addr));
         return;
     }
 
-    gnb_node_t *df_node = GNB_HASH32_UINT32_GET_PTR(gnb_core->uuid_node_map, df_uuid32);
+    gnb_node_t *df_node = GNB_HASH32_UINT64_GET_PTR(gnb_core->uuid_node_map, df_uuid64);
 
     if ( NULL==df_node ) {
-        GNB_LOG2(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_uf_node_notify_frame df node=%u is miss\n", df_uuid32);
+        GNB_LOG2(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_uf_node_notify_frame df node=%llu is miss\n", df_uuid64);
         return;
     }
 
-    GNB_LOG2(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_uf_node_notify_frame src_uuid32=%u dst_uuid32=%u df_uuid32=%u\n", src_uuid32, dst_uuid32, df_uuid32);
+    GNB_LOG2(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_uf_node_notify_frame src_uuid64=%llu dst_uuid64=%llu df_uuid64=%llu\n", src_uuid64, dst_uuid64, df_uuid64);
 
     update_unified_forwarding_node_array(gnb_core, df_node, src_node);
 
@@ -172,19 +171,19 @@ static void send_uf_node_notify_frame(gnb_core_t *gnb_core, gnb_node_t *dst_node
 
     memset(uf_node_notify_frame, 0, sizeof(uf_node_notify_frame_t));
 
-    uf_node_notify_frame->data.src_uuid32 = htonl(gnb_core->local_node->uuid32);
-    uf_node_notify_frame->data.dst_uuid32 = htonl(dst_node->uuid32);
-    uf_node_notify_frame->data.df_uuid32  = htonl(direct_forwarding_node->uuid32);
+    uf_node_notify_frame->data.src_uuid64 = gnb_htonll(gnb_core->local_node->uuid64);
+    uf_node_notify_frame->data.dst_uuid64 = gnb_htonll(dst_node->uuid64);
+    uf_node_notify_frame->data.df_uuid64  = gnb_htonll(direct_forwarding_node->uuid64);
 
-    snprintf((char *)uf_node_notify_frame->data.text, 64, "UNIFIED_NOTIFY src=%u dst=%u df=%u", gnb_core->local_node->uuid32, dst_node->uuid32, direct_forwarding_node->uuid32);
+    snprintf((char *)uf_node_notify_frame->data.text, 64, "UNIFIED_NOTIFY src=%llu dst=%llu df=%llu", gnb_core->local_node->uuid64, dst_node->uuid64, direct_forwarding_node->uuid64);
 
     if ( 0 == gnb_core->conf->lite_mode ) {
         ed25519_sign(uf_node_notify_frame->src_sign, (const unsigned char *)&uf_node_notify_frame->data, sizeof(struct uf_node_notify_frame_data), gnb_core->ed25519_public_key, gnb_core->ed25519_private_key);
     }
 
-    gnb_forward_payload_to_node(gnb_core, dst_node, node_worker_ctx->node_frame_payload);
+    gnb_p2p_forward_payload_to_node(gnb_core, dst_node, node_worker_ctx->node_frame_payload);
 
-    GNB_LOG2(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "send_uf_node_notify_frame src=%u notify=%u direct_forwarding=%u\n", gnb_core->local_node->uuid32, dst_node->uuid32, direct_forwarding_node->uuid32);
+    GNB_LOG2(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "send_uf_node_notify_frame src=%llu notify=%llu direct_forwarding=%llu\n", gnb_core->local_node->uuid64, dst_node->uuid64, direct_forwarding_node->uuid64);
 
 }
 
@@ -205,11 +204,11 @@ static void unifield_forwarding_notify(gnb_core_t *gnb_core, gnb_node_t *dst_nod
 
         df_node = &gnb_core->ctl_block->node_zone->node[i];
 
-        if ( dst_node->uuid32 == df_node->uuid32 ) {
+        if ( dst_node->uuid64 == df_node->uuid64 ) {
             continue;
         }
 
-        if ( gnb_core->local_node->uuid32 == df_node->uuid32 ) {
+        if ( gnb_core->local_node->uuid64 == df_node->uuid64 ) {
             continue;
         }
 
@@ -237,11 +236,9 @@ static void send_ping_frame(gnb_core_t *gnb_core, gnb_node_t *node){
     }
 
     if ( GNB_NODE_STATUS_UNREACHABL == node->udp_addr_status ) {
-
         if ( !(node->type & GNB_NODE_TYPE_IDX) && !(node->type & GNB_NODE_TYPE_FWD) && !(node->type & GNB_NODE_TYPE_RELAY) && !(node->type & GNB_NODE_TYPE_STATIC_ADDR) ) {
             return;
         }
-
     }
 
     node_worker_ctx->node_frame_payload->sub_type = PAYLOAD_SUB_TYPE_PING;
@@ -250,12 +247,12 @@ static void send_ping_frame(gnb_core_t *gnb_core, gnb_node_t *node){
     node_ping_frame_t *node_ping_frame = (node_ping_frame_t *)node_worker_ctx->node_frame_payload->data;
     memset(node_ping_frame, 0, sizeof(node_ping_frame_t));
 
-    node_ping_frame->data.src_uuid32 = htonl(gnb_core->local_node->uuid32);
-    node_ping_frame->data.dst_uuid32 = htonl(node->uuid32);
+    node_ping_frame->data.src_uuid64 = gnb_htonll(gnb_core->local_node->uuid64);
+    node_ping_frame->data.dst_uuid64 = gnb_htonll(node->uuid64);
 
     node_ping_frame->data.src_ts_usec = gnb_htonll(node_worker_ctx->now_time_usec);
 
-    snprintf((char *)node_ping_frame->data.text,32,"%d --PING-> %d", gnb_core->local_node->uuid32, node->uuid32);
+    snprintf((char *)node_ping_frame->data.text, 32, "%llu --PING-> %llu", gnb_core->local_node->uuid64, node->uuid64);
 
     if ( 0 == gnb_core->conf->lite_mode ) {
         ed25519_sign(node_ping_frame->src_sign, (const unsigned char *)&node_ping_frame->data, sizeof(struct ping_frame_data), gnb_core->ed25519_public_key, gnb_core->ed25519_private_key);
@@ -268,10 +265,10 @@ static void send_ping_frame(gnb_core_t *gnb_core, gnb_node_t *node){
     node->ping_ts_sec  = node_worker_ctx->now_time_sec;
     node->ping_ts_usec = node_worker_ctx->now_time_usec;
 
-    GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "ping src[%u]->dst[%u] %s %s ping_ts=%"PRIu64"\n",
-            gnb_core->local_node->uuid32, node->uuid32,
+    GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "ping src[%llu]->dst[%llu] %s %s ping_ts=%"PRIu64"\n",
+            gnb_core->local_node->uuid64, node->uuid64,
             GNB_SOCKADDR6STR1(&node->udp_sockaddr6), GNB_SOCKADDR4STR2(&node->udp_sockaddr4),
-            node->ping_ts_sec
+            node->ping_ts_usec
     );
 
 }
@@ -283,24 +280,24 @@ static void handle_ping_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *node_w
     node_worker_ctx_t *node_worker_ctx = gnb_core->node_worker->ctx;
     node_ping_frame_t *node_ping_frame = (node_ping_frame_t *)&node_worker_in_data->payload_st.data;
 
-    uint32_t src_uuid32 = ntohl(node_ping_frame->data.src_uuid32);
-    uint32_t dst_uuid32 = ntohl(node_ping_frame->data.dst_uuid32);
+    gnb_uuid_t src_uuid64 = gnb_ntohll(node_ping_frame->data.src_uuid64);
+    gnb_uuid_t dst_uuid64 = gnb_ntohll(node_ping_frame->data.dst_uuid64);
 
     //收到ping frame，需要更新node的addr
     gnb_sockaddress_t *node_addr = &node_worker_in_data->node_addr_st;
 
     uint8_t addr_update = 0;
 
-    if (src_uuid32 == gnb_core->local_node->uuid32) {
+    if (src_uuid64 == gnb_core->local_node->uuid64) {
         //目标节点和当前节点在同一个子网里，当前节点发 ping frame 经过路由的nat又转回给了当前节点
-        GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "handle_ping_frame local_node[%u] src[%u]=>dst[%u]\n", gnb_core->local_node->uuid32, src_uuid32, dst_uuid32);
+        GNB_LOG3(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_ping_frame local_node[%llu] src[%llu]=>dst[%llu]\n", gnb_core->local_node->uuid64, src_uuid64, dst_uuid64);
         return;
     }
 
-    gnb_node_t *src_node = GNB_HASH32_UINT32_GET_PTR(gnb_core->uuid_node_map, src_uuid32);
+    gnb_node_t *src_node = GNB_HASH32_UINT64_GET_PTR(gnb_core->uuid_node_map, src_uuid64);
 
     if ( NULL==src_node ) {
-        GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "handle_ping_frame node=%u is miss\n", src_uuid32);
+        GNB_LOG3(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_ping_frame error node=%llu is miss\n", src_uuid64);
         return;
     }
 
@@ -314,7 +311,7 @@ static void handle_ping_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *node_w
     }
 
     if ( 0 == gnb_core->conf->lite_mode && !ed25519_verify(node_ping_frame->src_sign, (const unsigned char *)&node_ping_frame->data, sizeof(struct ping_frame_data), src_node->public_key) ) {
-        GNB_LOG3(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_ping_frame invalid signature src=%u %s\n", src_uuid32, GNB_SOCKETADDRSTR1(node_addr));
+        GNB_LOG3(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_ping_frame error invalid signature src=%llu text[%.*s] %s\n", src_uuid64, 32, node_ping_frame->data.text, GNB_SOCKETADDRSTR1(node_addr));
         return;
     }
 
@@ -324,41 +321,55 @@ static void handle_ping_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *node_w
 
     if ( AF_INET6 == node_addr->addr_type ) {
 
-        src_node->udp_addr_status |= GNB_NODE_STATUS_IPV6_PING;
+        if ( 0 != gnb_determine_subnet6_prefixlen96(node_addr->addr.in6.sin6_addr, gnb_core->local_node->tun_ipv6_addr ) ) {
+
+            GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "handle_ping_frame IPV6 Warning src[%llu]->dst[%llu] idx=%u %s\n",
+                    src_node->uuid64, dst_uuid64,
+                    node_worker_in_data->socket_idx,
+                    GNB_ADDR6STR1(&node_addr->addr.in6.sin6_addr)
+            );
+
+            return;
+
+        }
 
         //只在发生改变的时候才更新
         if ( 0 != gnb_cmp_sockaddr_in6(&src_node->udp_sockaddr6, &node_addr->addr.in6) || node_worker_in_data->socket_idx != src_node->socket6_idx ) {
             src_node->udp_sockaddr6 = node_addr->addr.in6;
             src_node->socket6_idx   = node_worker_in_data->socket_idx;
-            addr_update = 1;
+            addr_update = 1;        
         }
 
+        src_node->udp_addr_status |= GNB_NODE_STATUS_IPV6_PING;
         src_node->addr6_update_ts_sec = node_worker_ctx->now_time_sec;
 
-        GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "handle_ping_frame IPV6 src[%u]->dst[%u] idx=%u %s now=%"PRIu64" src_ts=%"PRIu64" up=%u different=%"PRId64"\n",
-                src_node->uuid32, dst_uuid32,
+        GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "handle_ping_frame IPV6 src[%llu]->dst[%llu] idx=%u %s now=%"PRIu64" src_ts=%"PRIu64" up=%u different=%"PRId64"\n",
+                src_node->uuid64, dst_uuid64,
                 node_worker_in_data->socket_idx,
                 GNB_SOCKADDR6STR1(&src_node->udp_sockaddr6),
                 node_worker_ctx->now_time_usec, src_ts_usec, addr_update, latency_usec);
 
-
     }
 
     if ( AF_INET == node_addr->addr_type ) {
 
-        src_node->udp_addr_status |= GNB_NODE_STATUS_IPV4_PING;
+        if ( 0 != gnb_determine_subnet4(node_addr->addr.in.sin_addr,  gnb_core->local_node->tun_addr4, gnb_core->local_node->tun_netmask_addr4) ) {
 
+            GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "handle_ping_frame IPV4 Warning src[%llu]->dst[%llu] idx=%u %s\n",
+                    src_node->uuid64, dst_uuid64,
+                    node_worker_in_data->socket_idx,
+                    GNB_ADDR4STR1(&node_addr->addr.in.sin_addr)
+            );
+
+            return;
+            
+        }
+        
         //只在发生改变的时候才更新
         if ( PAYLOAD_SUB_TYPE_PING == node_worker_in_data->payload_st.sub_type && (0 != gnb_cmp_sockaddr_in(&src_node->udp_sockaddr4, &node_addr->addr.in) || node_worker_in_data->socket_idx != src_node->socket4_idx) ) {
             src_node->udp_sockaddr4 = node_addr->addr.in;
             src_node->socket4_idx   = node_worker_in_data->socket_idx;
             addr_update = 1;
-
-            GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "handle_ping_frame IPV4 src[%u]->dst[%u] idx=%u %s now=%"PRIu64" src_ts=%"PRIu64" up=%u different=%"PRId64"\n",
-                    src_node->uuid32, dst_uuid32,
-                    node_worker_in_data->socket_idx,
-                    GNB_SOCKADDR4STR1(&src_node->udp_sockaddr4),
-                    node_worker_ctx->now_time_usec, src_ts_usec, addr_update, latency_usec);
         }
 
         //处理来自 LAN 的 ping frame
@@ -371,15 +382,22 @@ static void handle_ping_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *node_w
             src_node->socket4_idx   = node_worker_in_data->socket_idx;
             addr_update = 1;
 
-            GNB_LOG3(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_ping_frame IPV4 LAN src[%u]->dst[%u] idx=%u %s now=%"PRIu64" src_ts=%"PRIu64" up=%u different=%"PRId64"\n",
-                    src_node->uuid32, dst_uuid32,
+            GNB_LOG3(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_ping_frame IPV4 LAN src[%llu]->dst[%llu] idx=%u %s now=%"PRIu64" src_ts=%"PRIu64" up=%u different=%"PRId64"\n",
+                    src_node->uuid64, dst_uuid64,
                     node_worker_in_data->socket_idx,
                     GNB_SOCKADDR4STR1(&src_node->udp_sockaddr4),
                     node_worker_ctx->now_time_usec, src_ts_usec, addr_update, latency_usec);
 
         }
 
+        src_node->udp_addr_status |= GNB_NODE_STATUS_IPV4_PING;
         src_node->addr4_update_ts_sec = node_worker_ctx->now_time_sec;
+        
+        GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "handle_ping_frame IPV4 src[%llu]->dst[%llu] idx=%u %s now=%"PRIu64" src_ts=%"PRIu64" up=%u different=%"PRId64"\n",
+                src_node->uuid64, dst_uuid64,
+                node_worker_in_data->socket_idx,
+                GNB_SOCKADDR4STR1(&src_node->udp_sockaddr4),
+                node_worker_ctx->now_time_usec, src_ts_usec, addr_update, latency_usec);
 
     }
 
@@ -391,8 +409,8 @@ static void handle_ping_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *node_w
 
     memset(node_pong_frame, 0, sizeof(node_pong_frame_t));
 
-    node_pong_frame->data.src_uuid32 = htonl(gnb_core->local_node->uuid32);
-    node_pong_frame->data.dst_uuid32 = htonl(src_node->uuid32);
+    node_pong_frame->data.src_uuid64 = gnb_htonll(gnb_core->local_node->uuid64);
+    node_pong_frame->data.dst_uuid64 = gnb_htonll(src_node->uuid64);
 
     node_pong_frame->data.src_ts_usec = gnb_htonll(node_worker_ctx->now_time_usec);
     node_pong_frame->data.dst_ts_usec = node_ping_frame->data.src_ts_usec;
@@ -406,7 +424,7 @@ static void handle_ping_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *node_w
     memcpy(&attachment_tun_sockaddress->tun_addr4, &gnb_core->local_node->tun_addr4.s_addr, 4);
     attachment_tun_sockaddress->tun_sin_port4 = gnb_core->local_node->tun_sin_port4;
 
-    snprintf((char *)node_pong_frame->data.text,32,"%d --PONG-> %d",gnb_core->local_node->uuid32,src_node->uuid32);
+    snprintf((char *)node_pong_frame->data.text,32,"%llu --PONG-> %llu",gnb_core->local_node->uuid64,src_node->uuid64);
 
     if ( 0 == gnb_core->conf->lite_mode ) {
         ed25519_sign(node_pong_frame->src_sign, (const unsigned char *)&node_pong_frame->data, sizeof(struct pong_frame_data), gnb_core->ed25519_public_key, gnb_core->ed25519_private_key);
@@ -434,8 +452,8 @@ static void handle_pong_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *node_w
 
     node_pong_frame_t *node_pong_frame = (node_pong_frame_t *)&node_worker_in_data->payload_st.data;
 
-    uint32_t src_uuid32 = ntohl(node_pong_frame->data.src_uuid32);
-    uint32_t dst_uuid32 = ntohl(node_pong_frame->data.dst_uuid32);
+    gnb_uuid_t src_uuid64 = gnb_ntohll(node_pong_frame->data.src_uuid64);
+    gnb_uuid_t dst_uuid64 = gnb_ntohll(node_pong_frame->data.dst_uuid64);
 
     uint64_t dst_ts_usec = gnb_ntohll(node_pong_frame->data.dst_ts_usec);
 
@@ -447,15 +465,15 @@ static void handle_pong_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *node_w
 
     uint8_t addr_update = 0;
 
-    if ( dst_uuid32 != gnb_core->local_node->uuid32 ) {
-        GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "handle_pong_frame dst_uuid32[%u] != local_node[%u] addr_type=%d\n", dst_uuid32, gnb_core->local_node->uuid32, node_worker_in_data->node_addr_st.addr_type);
+    if ( dst_uuid64 != gnb_core->local_node->uuid64 ) {
+        GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "handle_pong_frame dst_uuid64[%llu] != local_node[%llu] addr_type=%d\n", dst_uuid64, gnb_core->local_node->uuid64, node_worker_in_data->node_addr_st.addr_type);
         return;
     }
 
-    gnb_node_t *src_node = GNB_HASH32_UINT32_GET_PTR(gnb_core->uuid_node_map, src_uuid32);
+    gnb_node_t *src_node = GNB_HASH32_UINT64_GET_PTR(gnb_core->uuid_node_map, src_uuid64);
 
     if ( NULL==src_node ) {
-        GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "handle_pong_frame node=%u is miss\n", src_uuid32);
+        GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "handle_pong_frame error node=%llu is miss\n", src_uuid64);
         return;
     }
 
@@ -469,7 +487,7 @@ static void handle_pong_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *node_w
     }
 
     if ( 0 == gnb_core->conf->lite_mode && !ed25519_verify(node_pong_frame->src_sign, (const unsigned char *)&node_pong_frame->data, sizeof(struct pong_frame_data), src_node->public_key) ) {
-        GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER,"handle_pong_frame node=%u invalid signature\n", src_uuid32);
+        GNB_LOG3(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_pong_frame error node=%llu invalid signature text[%.*s] %s\n", src_uuid64, 32, node_pong_frame->data.text, GNB_SOCKETADDRSTR1(node_addr));
         return;
     }
 
@@ -481,6 +499,18 @@ static void handle_pong_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *node_w
 
     if ( AF_INET6 == node_addr->addr_type ) {
 
+        if ( 0 != gnb_determine_subnet6_prefixlen96(node_addr->addr.in6.sin6_addr, gnb_core->local_node->tun_ipv6_addr ) ) {
+
+            GNB_LOG3(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_pong_frame IPV6 Warning src[%llu]->dst[%llu] idx=%u %s\n",
+                    src_node->uuid64, dst_uuid64,
+                    node_worker_in_data->socket_idx,
+                    GNB_ADDR6STR1(&node_addr->addr.in6.sin6_addr)
+            );
+
+            return;
+
+        }
+
         src_node->udp_addr_status |= GNB_NODE_STATUS_IPV6_PONG;
 
         gnb_set_address6(&address_st, &node_addr->addr.in6);
@@ -502,13 +532,13 @@ static void handle_pong_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *node_w
 
             src_node->addr6_ping_latency_usec = node_worker_ctx->now_time_usec - dst_ts_usec + 1;
             if ( 0 == src_node->addr6_ping_latency_usec ) {
-                GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER,"addr6_ping_latency_usec==0 now=%"PRIu64" dst_ts=%"PRIu64"\n",node_worker_ctx->now_time_usec, dst_ts_usec);
+                GNB_LOG3(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "addr6_ping_latency_usec==0 now=%"PRIu64" dst_ts=%"PRIu64"\n", node_worker_ctx->now_time_usec, dst_ts_usec);
             }
 
         }
 
-        GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "handle_pong_frame IPV6 src[%u]->dst[%u] idx=%u %s now=%"PRIu64" dst_ts=%"PRIu64" up=%u latency=%"PRId64"\n",
-                src_node->uuid32, dst_uuid32,
+        GNB_LOG3(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_pong_frame IPV6 src[%llu]->dst[%llu] idx=%u %s now=%"PRIu64" dst_ts=%"PRIu64" up=%u latency=%"PRId64"\n",
+                src_node->uuid64, dst_uuid64,
                 node_worker_in_data->socket_idx,
                 GNB_SOCKADDR6STR1(&src_node->udp_sockaddr6),
                 node_worker_ctx->now_time_usec, dst_ts_usec, addr_update, src_node->addr6_ping_latency_usec);
@@ -518,6 +548,18 @@ static void handle_pong_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *node_w
 
     if ( AF_INET == node_addr->addr_type ) {
 
+        if ( 0 != gnb_determine_subnet4(node_addr->addr.in.sin_addr,  gnb_core->local_node->tun_addr4, gnb_core->local_node->tun_netmask_addr4) ) {
+
+            GNB_LOG3(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_pong_frame IPV4 Warning src[%llu]->dst[%llu] idx=%u %s\n",
+                    src_node->uuid64, dst_uuid64,
+                    node_worker_in_data->socket_idx,
+                    GNB_ADDR4STR1(&node_addr->addr.in.sin_addr)
+            );
+
+            return;
+            
+        }
+
         src_node->udp_addr_status |= GNB_NODE_STATUS_IPV4_PONG;
 
         gnb_set_address4(&address_st, &node_addr->addr.in);
@@ -539,20 +581,19 @@ static void handle_pong_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *node_w
 
             src_node->addr4_ping_latency_usec = node_worker_ctx->now_time_usec - dst_ts_usec  + 1;
             if ( 0 == src_node->addr4_ping_latency_usec ) {
-                GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER,"addr4_ping_latency_usec==0 now=%"PRIu64" dst_ts=%"PRIu64"\n",node_worker_ctx->now_time_usec, dst_ts_usec);
+                GNB_LOG3(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "addr4_ping_latency_usec=0 now=%"PRIu64" dst_ts=%"PRIu64"\n", node_worker_ctx->now_time_usec, dst_ts_usec);
             }
 
         }
 
-        GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "handle_pong_frame IPV4 src[%u]->dst[%u] idx=%u %s now=%"PRIu64" dst_ts=%"PRIu64" up=%u latency=%"PRId64"\n",
-                src_node->uuid32, dst_uuid32,
+        GNB_LOG3(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "handle_pong_frame IPV4 src[%llu]->dst[%llu] idx=%u %s now=%"PRIu64" dst_ts=%"PRIu64" up=%u latency=%"PRId64"\n",
+                src_node->uuid64, dst_uuid64,
                 node_worker_in_data->socket_idx,
                 GNB_SOCKADDR4STR1(&src_node->udp_sockaddr4),
                 node_worker_ctx->now_time_usec, dst_ts_usec, addr_update, src_node->addr4_ping_latency_usec);
 
     }
 
-
     //处理附件
     gnb_payload16_t *payload_attachment = (gnb_payload16_t *)node_pong_frame->data.attachment;
 
@@ -570,8 +611,8 @@ static void handle_pong_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *node_w
 
     if ( PAYLOAD_SUB_TYPE_PONG2 == node_worker_in_data->payload_st.sub_type ) {
 
-        GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "handle_pong2_frame  src[%u]->dst[%u] idx=%u\n",
-                src_node->uuid32, dst_uuid32,
+        GNB_LOG3(gnb_core->log,GNB_LOG_ID_NODE_WORKER, "handle_pong2_frame  src[%llu]->dst[%llu] idx=%u\n",
+                src_node->uuid64, dst_uuid64,
                 node_worker_in_data->socket_idx);
 
         return;
@@ -585,8 +626,8 @@ static void handle_pong_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *node_w
 
     memset(node_pong2_frame,0,sizeof(node_pong_frame_t));
 
-    node_pong2_frame->data.src_uuid32 = htonl(gnb_core->local_node->uuid32);
-    node_pong2_frame->data.dst_uuid32 = htonl(src_node->uuid32);
+    node_pong2_frame->data.src_uuid64 = gnb_htonll(gnb_core->local_node->uuid64);
+    node_pong2_frame->data.dst_uuid64 = gnb_htonll(src_node->uuid64);
 
     node_pong2_frame->data.src_ts_usec = gnb_htonll(node_worker_ctx->now_time_usec);
     node_pong2_frame->data.dst_ts_usec = node_pong2_frame->data.src_ts_usec;
@@ -595,7 +636,7 @@ static void handle_pong_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *node_w
     gnb_payload16_set_data_len(payload_attachment2, 0);
     payload_attachment2->type = GNB_NODE_ATTACHMENT_TYPE_TUN_EMPTY;
 
-    snprintf((char *)node_pong2_frame->data.text,32,"%d --PONG2-> %d",gnb_core->local_node->uuid32,src_node->uuid32);
+    snprintf((char *)node_pong2_frame->data.text,32,"%llu --PONG2-> %llu",gnb_core->local_node->uuid64,src_node->uuid64);
 
     if ( 0 == gnb_core->conf->lite_mode ) {
         ed25519_sign(node_pong2_frame->src_sign, (const unsigned char *)&node_pong2_frame->data, sizeof(struct pong_frame_data), gnb_core->ed25519_public_key, gnb_core->ed25519_private_key);
@@ -617,43 +658,6 @@ static void handle_pong_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *node_w
 }
 
 
-static void update_node_crypto_key(gnb_core_t *gnb_core, uint64_t now_sec){
-
-    int need_update_time_seed;
-
-    size_t num = gnb_core->ctl_block->node_zone->node_num;
-
-    if ( 0 == num ) {
-        return;
-    }
-
-    need_update_time_seed = gnb_verify_seed_time(gnb_core, now_sec);
-
-    if ( 0 == need_update_time_seed ) {
-        return;
-    }
-
-    gnb_update_time_seed(gnb_core, now_sec);
-
-    int i;
-
-    gnb_node_t *node;
-
-    for ( i=0; i<num; i++ ) {
-
-        node = &gnb_core->ctl_block->node_zone->node[i];
-
-        if ( gnb_core->local_node->uuid32 == node->uuid32 ) {
-            continue;
-        }
-
-        gnb_build_crypto_key(gnb_core, node);
-
-    }
-
-}
-
-
 static void sync_node(gnb_worker_t *gnb_node_worker){
 
     node_worker_ctx_t *node_worker_ctx = gnb_node_worker->ctx;
@@ -674,7 +678,7 @@ static void sync_node(gnb_worker_t *gnb_node_worker){
 
         node = &gnb_core->ctl_block->node_zone->node[i];
 
-        if ( gnb_core->local_node->uuid32 == node->uuid32 ) {
+        if ( gnb_core->local_node->uuid64 == node->uuid64 ) {
             continue;
         }
 
@@ -706,14 +710,15 @@ static void sync_node(gnb_worker_t *gnb_node_worker){
             send_ping_frame(gnb_core, node);
         }
 
-        if ( (node_worker_ctx->now_time_sec - node->last_notify_uf_nodes_ts_sec) >= GNB_UF_NODES_NOTIFY_INTERVAL_SEC ) {
+        if (  GNB_UNIFIED_FORWARDING_OFF != gnb_core->conf->unified_forwarding && (node_worker_ctx->now_time_sec - node->last_notify_uf_nodes_ts_sec) >= GNB_UF_NODES_NOTIFY_INTERVAL_SEC ) {
 
             if ( !( (GNB_NODE_STATUS_IPV6_PONG | GNB_NODE_STATUS_IPV4_PONG) & node->udp_addr_status ) ) {
                 continue;
             }
 
-            GNB_LOG3(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "unifield_forwarding_notify nodeid=%u\n", node->uuid32);
+            GNB_LOG3(gnb_core->log, GNB_LOG_ID_NODE_WORKER, "unifield_forwarding_notify nodeid=%llu\n", node->uuid64);
             unifield_forwarding_notify(gnb_core, node);
+
             node->last_notify_uf_nodes_ts_sec = node_worker_ctx->now_time_sec;
 
         }
@@ -729,7 +734,7 @@ static void sync_node(gnb_worker_t *gnb_node_worker){
 
         if ( (node_worker_ctx->now_time_sec - node->addr6_update_ts_sec) > GNB_NODE_UPDATE_INTERVAL_SEC ) {
             //节点状态超时，且不是idx node, 可能目标node已经下线或者更换了ip
-            if ( !(node->type & GNB_NODE_TYPE_IDX) ){
+            if ( !(node->type & GNB_NODE_TYPE_IDX) ) {
                 node->udp_addr_status &= ~(GNB_NODE_STATUS_IPV6_PONG | GNB_NODE_STATUS_IPV6_PING);
             }
 
@@ -753,18 +758,15 @@ static void handle_node_frame(gnb_core_t *gnb_core, gnb_worker_in_data_t *node_w
 
         case PAYLOAD_SUB_TYPE_PING:
         case PAYLOAD_SUB_TYPE_LAN_PING:
-
             handle_ping_frame(gnb_core, node_worker_in_data);
             break;
 
         case PAYLOAD_SUB_TYPE_PONG  :
         case PAYLOAD_SUB_TYPE_PONG2 :
-
             handle_pong_frame(gnb_core, node_worker_in_data);
             break;
 
         case PAYLOAD_SUB_TYPE_NODE_UNIFIED_NOTIFY:
-
             handle_uf_node_notify_frame(gnb_core, node_worker_in_data);
             break;
 
@@ -858,7 +860,7 @@ static void init(gnb_worker_t *gnb_worker, void *ctx){
     node_worker_ctx_t *node_worker_ctx =  (node_worker_ctx_t *)gnb_heap_alloc(gnb_core->heap, sizeof(node_worker_ctx_t));
     memset(node_worker_ctx, 0, sizeof(node_worker_ctx_t));
     node_worker_ctx->gnb_core = gnb_core;
-    node_worker_ctx->node_frame_payload =  (gnb_payload16_t *)gnb_heap_alloc(gnb_core->heap,1600);
+    node_worker_ctx->node_frame_payload =  (gnb_payload16_t *)gnb_heap_alloc(gnb_core->heap, gnb_core->conf->payload_block_size);
     node_worker_ctx->node_frame_payload->type = GNB_PAYLOAD_TYPE_NODE;
 
     memory_size = gnb_ring_buffer_fixed_sum_size(GNB_NODE_WORKER_QUEUE_BLOCK_SIZE, gnb_core->conf->node_woker_queue_length);
@@ -907,18 +909,13 @@ static int stop(gnb_worker_t *gnb_worker){
 
 
 static int notify(gnb_worker_t *gnb_worker){
-
     int ret;
-
     node_worker_ctx_t *node_worker_ctx = gnb_worker->ctx;
     ret = pthread_kill(node_worker_ctx->thread_worker,SIGALRM);
-
     return 0;
-
 }
 
 gnb_worker_t gnb_node_worker_mod = {
-
     .name      = "gnb_node_worker",
     .init      = init,
     .release   = release,
@@ -926,5 +923,4 @@ gnb_worker_t gnb_node_worker_mod = {
     .stop      = stop,
     .notify    = notify,
     .ctx       = NULL
-
 };
